#pragma once

#include <QString>
#include <QList>
#include <functional>
#include "cmds_basic.h"

namespace validate {
	template<typename T> class Validator {
	public:
		virtual QString name() const {
			return QString();
		}
		virtual bool check(T value) const {
			return false;
		}
		virtual void getDepict(datas::IDataObject& self_description) { }
	};
	template<typename T> class ValidatorReceiver {
	public:
		virtual void append(Validator<T>* inst) { }
	};




	template<typename... Args> struct ValidateTemplet;
	template<> struct ValidateTemplet<> {
		template<typename Ret>
		bool validateFor(Inlet::SignatureImpl<Ret>& value_sequence) const {
			return true;
		}

		virtual void getDepict(datas::IDataObject& self_description) const {}
	};
	template<typename Head, typename... Rest>
	struct ValidateTemplet<Head, Rest...> : public ValidateTemplet<Rest...> {
	private:
		QList<Validator<Head>*> validator_list;

	public:
		void add(Validator<Head>* vinst) {
			validator_list << vinst;
		}
		bool check(Head value) const {
			for (auto u : validator_list)
				if (!u->check(value))
					return false;
			return true;
		}

		template<typename Ret>
		bool validateFor(Inlet::SignatureImpl<Ret, Head, Rest...>& value_sequence) const {
			Head head_value = value_sequence.getArgs0();
			if (!check(head_value))
				return false;
			return ValidateTemplet<Rest...>::validateFor(value_sequence);
		}


		virtual void getDepict(datas::IDataObject& self_description) const {
			auto args_head = self_description.newObject();
			for (auto vinst : validator_list) {
				auto vdesc = self_description.newObject();
				vinst->getDepict(*vdesc);
				args_head->setChild(vinst->name(), vdesc);
			}

			auto args_array = std::dynamic_pointer_cast<datas::IDataArray>(self_description.getChild(u8"ArgsList"));
			args_array->append(args_head);
			self_description.setChild(u8"ArgsList", args_array);
			ValidateTemplet<Rest...>::getDepict(self_description);
		}
	};




	template<int idx, typename... Args> struct ElmPos;

	template<typename Head, typename... Rest>
	struct ElmPos<0, ValidateTemplet<Head, Rest...>> {
		using value_type = Head;
		using templet_type = ValidateTemplet<Head, Rest...>;
	};
	template<int idx, typename Head, typename... Rest>
	struct ElmPos<idx, ValidateTemplet<Head, Rest...>> {
		using value_type = typename ElmPos<idx - 1, ValidateTemplet<Rest...>>::value_type;
		using templet_type = typename ElmPos<idx - 1, ValidateTemplet<Rest...>>::templet_type;
	};

	template<int idx, typename Head, typename... Rest>
	void __validator_insert_helper(ValidateTemplet<Head, Rest...>& target,
		Validator<typename ElmPos<idx, ValidateTemplet<Head, Rest...>>::value_type>* inst) {
		using templet_type = typename ElmPos<idx, ValidateTemplet<Head, Rest...>>::templet_type;
		target.templet_type::add(inst);
	}




	template<int n, typename... Args> class SlicePosition;

	template<typename S, typename T> class SliceValidator : public Validator<S> {
	private:
		std::function<T(S)> _proc_conv;
		Validator<T> _sub_validator;

	public:
		SliceValidator(std::function<T(S)> proc, Validator<T> vinst)
			: _proc_conv(proc), _sub_validator(vinst) { }

		virtual bool check(S value) const {
			auto t_value = _proc_conv(value);
			return _sub_validator.check(t_value);
		}
	};
	template<int n, typename... Args> class ArgsPosition
		: public ValidatorReceiver<typename ElmPos<n, ValidateTemplet<Args...>>::value_type> {
	public:
		using Param = typename ElmPos<n, ValidateTemplet<Args...>>::value_type;

		ArgsPosition(ValidateTemplet<Args...>& target) :_templet_bind(target) { }

		ArgsPosition<n, Args...>& operator&(Validator<Param>* vinst) {
			append(vinst);
			return *this;
		}

		template<typename Slice>
		SlicePosition<n, Param, Slice, ArgsPosition<n, Args...>>
			slice(std::function<Slice(Param)> slice_fun) {
			return SlicePosition<n, Param, Slice, ArgsPosition<n, Args...>>(*this, slice_fun);
		}

		// ValidatorReceiver
		virtual void append(Validator<Param>* inst) {
			__validator_insert_helper<n, Args...>(_templet_bind, inst);
		}

	private:
		ValidateTemplet<Args...>& _templet_bind;
	};
	template<int n, typename S, typename T, typename... Args>
	class SlicePosition<n, S, T, ArgsPosition<n, Args...>> : public ValidatorReceiver<T> {
	public:
		SlicePosition(ValidatorReceiver<S>& host, std::function<T(S)> func)
			:_conv_func(func), _prev_bind(host) { }

		SlicePosition<n, S, T, ArgsPosition<n, Args...>>& operator&(Validator<T>* vinst) {
			append(vinst);
			return *this;
		}

		template<typename Slice>
		SlicePosition<n, T, Slice, ArgsPosition<n, Args...>>
			slice(std::function<Slice(T)> slice_fun) {
			return SlicePosition<n, T, Slice, ArgsPosition<n, Args...>>(*this, slice_fun);
		}

		// ValidatorReceiver
		virtual void append(Validator<T>* inst) {
			SliceValidator<S, T> mid_validator(_conv_func, inst);
			_prev_bind.append(mid_validator);
		}

	private:
		std::function<T(S)> _conv_func;
		ValidatorReceiver<S>& _prev_bind;
	};




	class IValidatorTemplet {
	public:
		virtual bool doValidate(Inlet::IRunbase* inst) const = 0;
		virtual void getDepict(datas::IDataObject& self_description) const = 0;
	};

	template<void* func, typename... types> struct ValidateImpls;
	template<void* func, typename Ret, typename... Args>
	struct ValidateImpls<func, Ret, Args...> : public ValidateTemplet<Args...>, public IValidatorTemplet {
		quint64 address() const {
			return (quint64) func;
		}

		template<int n> ArgsPosition<n, Args...> pos() {
			return ArgsPosition<n, Args...>(*this);
		}

		// Ivalidatortemplet
		virtual bool doValidate(Inlet::IRunbase* inst) const {
			auto valid_data = dynamic_cast<Inlet::CmdsImpl<func, Ret, Args...>*>(inst);
			return ValidateTemplet<Args...>::validateFor(valid_data->getArgSequence());
		}

		virtual void getDepict(datas::IDataObject& self_description) const {
			self_description.setString(u8"ValidateFor", QString::number((qulonglong)func));

			auto array_l = self_description.newArray();
			self_description.setChild(u8"ArgsList", array_l);
			ValidateTemplet<Args...>::getDepict(self_description);
		}
	};
}
